# Clean the workspace and invoke useful libraries
rm(list=ls())
library(rjags)

simu <- "simu_e"

# Define working directory
setwd("Z:/Shared/SH_LiQ/JSM_2016/CRM - TwoPatients")

###################################################################################################################
# This program simulates M trials for dose finding of combination of two agents. It uses the logistic model
# logit(p)=alpha+beta*x+gamma*y + eta*x*y.
# Dose levels x and y are standardized to be between 0 and 1.
# The model is reparameterized in terms of rho00 = probability of DLT when x=y=0,
# rho01 = probability of DLT when when x=0 y=1, rho10 = probability of DLT when x = 1 y = 0, and eta = interaction term between # drugs.
# Feasibility bound alpha is variable
# MTD is truncated to interval (-Xmin / (Xmax - Xmin), infinity) where Xmin and Xmax are actual dose levels of both agent A and # agent B.
# A dose escalation jump is restricted to be no more than delta1 = 20% or 25% of dose range.
###################################################################################################################
# This function simulates a binary DLT response.
resplogistcomb<-function(rho00,rho01,rho10,eta,theta,x,y){
 alpha<-log(rho00/(1-rho00))
 beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
 gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
 p<-1/(1+exp(-alpha-beta*x-gamma*y-eta*x*y))
 u<-runif(1)
 if (u < p)
 z<-1
 else
 z<-0
 z}

# This function computes the probability of DLT at dose combination (x,y)

pdlt<-function(rho00,rho01,rho10,eta,x,y){
alpha<-log(rho00/(1-rho00))
 beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
 gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
p<-1/(1+exp(-alpha-beta*x-gamma*y-eta*x*y))
p}

# This function rounds a continuous dose to the nearest discrete dose

rndnearest<-function(w,x){
  l<-length(x)
  mid<-numeric()
  for(i in 1:(l-1)){
    mid[i]<-(x[i]+x[i+1])/2}
  if (w==1) {
    y<-1} else 
      if (x[1] <= w & w < mid[1]) {
        y<-x[1]} else
          if (w >= mid[l-1]) {
            y<-x[l]} else
              
            {
              
              for (i in 2:(l-1)){
                if (mid[i-1] <= w & w <mid[i])
                  y<- x[i] } }
  y}

# This function finds the set of MTD
twodimmtd2<-function(rho00,rho01,rho10,eta,theta,x){
alpha<-log(rho00/(1-rho00))
beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
a<-(log(theta/(1-theta))-alpha-beta*x)/(gamma+eta*x)
a}

#Stop rule
Xlow<-0
Ylow<-0

Xmin<-100
Xmax<-500
lb<--Xmin/(Xmax - Xmin)

# Threshold for posterior probability of toxicity at minimum dose for stopping trial
delta<-0.8

# Number of Trials
M<-1000
# True parameters
source(paste0("./Informative/discrete/", simu, "/trueparameters.r"))

# Target Probability of DLT 
theta<-0.2

# Feasibility bound
alpha<-numeric()
alphaa<-seq(0.25,0.5,by=0.05)
for (ind in 1:5){
alpha[ind]<-alphaa[ind]
}
for (ind in 6:(NN-1)){
alpha[ind]<-alphaa[6]
}
# Prior parameters for rho00, rho01, rho10, eta
a00<-2.17
b00<-20.7
a01<-1.45
b01<-5.16
a10<-1.21
b10<-0.91
a<-1
b<-1
vrho00<-numeric()
veta<-numeric()
vrho01<-numeric()
vrho10<-numeric()

# mcmc parameters

chains<-1
burn<-10000
mm<-5000

# Declaration of intermediate statistics
trcmtdx1<-numeric()
trcmtdx2<-numeric()
trcmtdy1<-numeric()
trcmtdy1<-numeric()

# Declaration of the output statistics
dosex<-mat.or.vec(M,NN)
dosey<-mat.or.vec(M,NN)
dlt<-mat.or.vec(M,NN)
postlow<-mat.or.vec(M,NN/2)
postdlts<-mat.or.vec(M*nx*ny, 4)
temp1<-numeric()
temp2<-numeric()

aux <- 1

# Start of main loop for the trials

for (kk in 1:M) {

# data for first cohort of two patients- Z is DLT status


X<-c(0,0)
Y<-c(0,0)

# Simulate responses for patients 1 and 2 from logistic model

zz1 <- rbinom(1,1,grid$p[grid$x==X[1] & grid$y==Y[1]])
zz2 <- rbinom(1,1,grid$p[grid$x==X[2] & grid$y==Y[2]])
Z<-c(zz1,zz2)
n<-2

# Get posterior median of model parameters

j=jags.model('combination_cohort_2.bug.txt',data=list('Z'=Z,'X'=X,'Y'=Y,'a00'=a00,'b00'=b00,'a01'=a01,'b01'=b01,'a10'=a10,'b10'=b10,'a'=a,'b'=b,'N'=n),n.chains=chains,n.adapt=burn)

s=coda.samples(j,c('rho00','rho01','rho10','eta'),mm)
ss=as.data.frame(s[[1]])

medrh00<-median(ss$rho00)
medrh01<-median(ss$rho01)
medrh10<-median(ss$rho10)
medeta<-median(ss$eta)

####

#Find dose xx1 in interval [0,1] so that |pdlt(medrho00,medrho01,medrho10,medeta,xx1,Y[1]) - theta| is smallest
xx1 = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,Y[1]) - theta)}, c(0,1),tol=0.0001)$minimum

if ((xx1 - X[1]) > delta1x)
xx1<-X[1]+delta1x
xx1<-rndnearest(xx1,grid$x)

#Find dose yy1 in interval [0,1] so that |pdlt(medrho00,medrho01,medrho10,medeta,X[2],yy1) - theta| is smallest
yy2 = optimize(f=function(y){abs(pdlt(medrh00,medrh01,medrh10,medeta,X[2],y) - theta)}, c(0,1),tol=0.0001)$minimum

if ((yy2 - Y[2]) > delta1y)
yy2<-Y[2]+delta1y
yy2<-rndnearest(yy2,grid$y)

# Update data

X<-c(X,0,xx1)
Y<-c(Y,yy2,0)

# Simulate responses for patients 3 and 4 from logistic model

zz1 <- rbinom(1,1,grid$p[grid$x==X[3] & grid$y==Y[3]])
zz2 <- rbinom(1,1,grid$p[grid$x==X[4] & grid$y==Y[4]])
Z<-c(Z,zz1,zz2)
n<-4

for (i in 3:(NN/2)) {
j=jags.model('combination_cohort_2.bug.txt',data=list('Z'=Z,'X'=X,'Y'=Y,'a00'=a00,'b00'=b00,'a01'=a01,'b01'=b01,'a10'=a10,'b10'=b10,'a'=a,'b'=b,'N'=n),n.chains=chains,n.adapt=burn)

s=coda.samples(j,c('rho00','rho01','rho10','eta'),mm)
ss=as.data.frame(s[[1]])

# Calculating posterior probability of toxicity at initial dose
for (ll in 1:mm) {

temp1[ll]<-pdlt(ss$rho00[ll],ss$rho01[ll],ss$rho10[ll],ss$eta[ll],Xlow, Ylow)

if (temp1[ll] > theta+0.05)
(temp2[ll]<-1) else
(temp2[ll]<-0)
}
postlow[kk,i]<-sum(temp2)/mm


medrh00<-median(ss$rho00)
medrh01<-median(ss$rho01)
medrh10<-median(ss$rho10)
medeta<-median(ss$eta)


##
xx1 = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,Y[2*i-3]) - theta)}, c(0,1),tol=0.0001)$minimum
if ((xx1 - X[2*i-3]) > delta1x)
xx1<-X[2*i-3]+delta1x
xx1<-rndnearest(xx1,grid$x)

xx2 = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,Y[2*i-2]) - theta)}, c(0,1),tol=0.0001)$minimum
if ((xx2 - X[2*i-2]) > delta1x)
xx2<-X[2*i-2]+delta1x
xx2<-rndnearest(xx2,grid$x)

yy1 = optimize(f=function(y){abs(pdlt(medrh00,medrh01,medrh10,medeta,X[2*i-3],y) - theta)}, c(0,1),tol=0.0001)$minimum
if ((yy1 - Y[2*i-3]) > delta1y)
yy1<-Y[2*i-3]+delta1y
yy1<-rndnearest(yy1,grid$y)

yy2 = optimize(f=function(y){abs(pdlt(medrh00,medrh01,medrh10,medeta,X[2*i-2],y) - theta)}, c(0,1),tol=0.0001)$minimum
if ((yy2 - Y[2*i-2]) > delta1y)
yy2<-Y[2*i-2]+delta1y
yy2<-rndnearest(yy2,grid$y)

if (X[2*i-3] == X[2*i-5]) {

X<-c(X,xx1,X[2*i-2])
Y<-c(Y,Y[2*i-3],yy2)} else {
X<-c(X,X[2*i-3],xx2)
Y<-c(Y,yy1,Y[2*i-2])}


n<-n+2
zz1 <- rbinom(1,1,grid$p[grid$x==X[2*i-1] & grid$y==Y[2*i-1]])
zz2 <- rbinom(1,1,grid$p[grid$x==X[2*i] & grid$y==Y[2*i]])
Z<-c(Z,zz1,zz2)}

dosex[kk,]<-X
dosey[kk,]<-Y
dlt[kk,]<-Z


vrho00[kk]<-median(ss$rho00)
vrho01[kk]<-median(ss$rho01)
vrho10[kk]<-median(ss$rho10)
veta[kk]<-median(ss$eta)

for (i in 1:nrow(grid)) {
  temp <- pdlt(ss$rho00,ss$rho01,ss$rho10, ss$eta, grid$x[i], grid$y[i])
  postdlts[aux, ] <- c(grid$x[i], grid$y[i], 
                       mean(temp > theta + 0.1  | temp < theta - 0.1), kk)
  aux <- aux + 1
}


}

colnames(postdlts) <- c("xx", "yy", "potsdlt", "trial")
setwd(paste0("Z:/Shared/SH_LiQ/JSM_2016/CRM - TwoPatients/Informative/discrete/", 
             simu, "/"))
# output
write.table(vrho00,"rho00.txt")
write.table(vrho01,"rho01.txt")
write.table(vrho10,"rho10.txt")
write.table(veta,"eta.txt")
write.table(dosex,"dosex.txt")
write.table(dosey,"dosey.txt")
write.table(dlt,"dlt.txt")
write.table(postlow,"postlow.txt")
write.csv(postdlts,"postdlts.csv")

save.image("results.Rdata")
