
# Define working directory
setwd("M:/Shared/SH_LiQ/JSM_2016/TwoPatients")

# Clean the workspace and invoke useful libraries
rm(list=ls())
library(rjags)
library(plotrix)

###################################################################################################################
# This program simulates M trials for dose finding of combination of two agents. It uses the logistic model
# logit(p)=alpha+beta*x+gamma*y + eta*x*y.
# Dose levels x and y are standardized to be between 0 and 1.
# The model is reparameterized in terms of rho00 = probability of DLT when x=y=0,
# rho01 = probability of DLT when when x=0 y=1, rho10 = probability of DLT when x = 1 y = 0, and eta = interaction term between # drugs.
# Feasibility bound alpha is variable
# MTD is truncated to interval (-Xmin / (Xmax - Xmin), infinity) where Xmin and Xmax are actual dose levels of both agent A and # agent B.
# A dose escalation jump is restricted to be no more than delta1 = 20% or 25% of dose range.
###################################################################################################################
# This function simulates a binary DLT response.
resplogistcomb<-function(rho00,rho01,rho10,eta,theta,x,y){
 alpha<-log(rho00/(1-rho00))
 beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
 gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
 p<-1/(1+exp(-alpha-beta*x-gamma*y-eta*x*y))
 u<-runif(1)
 if (u < p)
 z<-1
 else
 z<-0
 z}

resp_loglog<-function(rho00,rho01,rho10,eta,theta,x,y){
  alpha<-log(rho00/(1-rho00))
  beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
  gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
  alpha1<-alpha-log(theta/(1-theta))+log(-log(1-theta))
  p<-1-exp(-exp(alpha1+beta*x+gamma*y+eta*x*y))
  u<-runif(1)
  if (u < p)
    z<-1
  else
    z<-0
  z}

# This function computes the probability of DLT at dose combination (x,y)

pdlt<-function(rho00,rho01,rho10,eta,x,y){
alpha<-log(rho00/(1-rho00))
 beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
 gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
p<-1/(1+exp(-alpha-beta*x-gamma*y-eta*x*y))
p}



# This function finds the set of MTD
twodimmtd2<-function(rho00,rho01,rho10,eta,theta,x){
alpha<-log(rho00/(1-rho00))
beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
a<-(log(theta/(1-theta))-alpha-beta*x)/(gamma+eta*x)
a}

delta1<-0.2
Xlow<-0
Ylow<-0

Xmin<-100
Xmax<-500
lb<--Xmin/(Xmax - Xmin)

# Threshold for posterior probability of toxicity at minimum dose for stopping trial
delta<-0.8

# Number of Trials
M<-1000
# True parameters
source("./continuous/simu_d/trueparameters.r")

# Target Probability of DLT 
theta<-0.33333
# Number of patients in a trial
NN<-40
# Feasibility bound
alpha<-numeric()
alphaa<-seq(0.25,0.5,by=0.05)
for (ind in 1:5){
alpha[ind]<-alphaa[ind]
}
for (ind in 6:(NN-1)){
alpha[ind]<-alphaa[6]
}
# Prior parameters for rho00, rho01, rho10, eta
a00<-1
b00<-1
a01<-1
b01<-1
a10<-1
b10<-1
# Prior parameters for interaction term eta
a<-0.8
b<-0.0384
vrho00<-numeric()
veta<-numeric()
vrho01<-numeric()
vrho10<-numeric()

# mcmc parameters

chains<-1
burn<-10000
mm<-5000

# Declaration of intermediate statistics
trcmtdx1<-numeric()
trcmtdx2<-numeric()
trcmtdy1<-numeric()
trcmtdy1<-numeric()

# Declaration of the output statistics
dosex<-mat.or.vec(M,NN)
dosey<-mat.or.vec(M,NN)
dlt<-mat.or.vec(M,NN)
postlow<-mat.or.vec(M,NN/2)
temp1<-numeric()
temp2<-numeric()

# Start of main loop for the trials

for (kk in 1:M) {

# data for first cohort of two patients- Z is DLT status


X<-c(0,0)
Y<-c(0,0)

# Simulate responses for patients 1 and 2 from logistic model

zz1<-resp_loglog(trho00,trho01,trho10,teta,theta,X[1],Y[1])
zz2<-resp_loglog(trho00,trho01,trho10,teta,theta,X[2],Y[2])
Z<-c(zz1,zz2)
n<-2

# Get posterior median of model parameters

j=jags.model('combination_cohort_2.bug.txt',data=list('Z'=Z,'X'=X,'Y'=Y,'a00'=a00,'b00'=b00,'a01'=a01,'b01'=b01,'a10'=a10,'b10'=b10,'a'=a,'b'=b,'N'=n),n.chains=chains,n.adapt=burn)

s=coda.samples(j,c('rho00','rho01','rho10','eta'),mm)
ss=as.data.frame(s[[1]])

medrh00<-median(ss$rho00)
medrh01<-median(ss$rho01)
medrh10<-median(ss$rho10)
medeta<-median(ss$eta)

####

#Find dose xx1 in interval [0,1] so that |pdlt(medrho00,medrho01,medrho10,medeta,xx1,Y[1]) - theta| is smallest
xx1 = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,Y[1]) - theta)}, c(0,1),tol=0.0001)$minimum

if ((xx1 - X[1]) > delta1)
xx1<-X[1]+delta1

#Find dose yy1 in interval [0,1] so that |pdlt(medrho00,medrho01,medrho10,medeta,X[2],yy1) - theta| is smallest
yy2 = optimize(f=function(y){abs(pdlt(medrh00,medrh01,medrh10,medeta,X[2],y) - theta)}, c(0,1),tol=0.0001)$minimum

if ((yy2 - Y[2]) > delta1)
yy2<-Y[2]+delta1

# Update data

X<-c(X,0,xx1)
Y<-c(Y,yy2,0)

# Simulate responses for patients 3 and 4 from logistic model

zz1<-resp_loglog(trho00,trho01,trho10,teta,theta,X[3],Y[3])
zz2<-resp_loglog(trho00,trho01,trho10,teta,theta,X[4],Y[4])
Z<-c(Z,zz1,zz2)
n<-4

for (i in 3:(NN/2)) {
j=jags.model('combination_cohort_2.bug.txt',data=list('Z'=Z,'X'=X,'Y'=Y,'a00'=a00,'b00'=b00,'a01'=a01,'b01'=b01,'a10'=a10,'b10'=b10,'a'=a,'b'=b,'N'=n),n.chains=chains,n.adapt=burn)

s=coda.samples(j,c('rho00','rho01','rho10','eta'),mm)
ss=as.data.frame(s[[1]])

# Calculating posterior probability of toxicity at initial dose
for (ll in 1:mm) {

temp1[ll]<-pdlt(ss$rho00[ll],ss$rho01[ll],ss$rho10[ll],ss$eta[ll],Xlow, Ylow)

if (temp1[ll] > theta+0.05)
(temp2[ll]<-1) else
(temp2[ll]<-0)
}
postlow[kk,i]<-sum(temp2)/mm


medrh00<-median(ss$rho00)
medrh01<-median(ss$rho01)
medrh10<-median(ss$rho10)
medeta<-median(ss$eta)


##
xx1 = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,Y[2*i-3]) - theta)}, c(0,1),tol=0.0001)$minimum
if ((xx1 - X[2*i-3]) > delta1)
xx1<-X[2*i-3]+delta1

xx2 = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,Y[2*i-2]) - theta)}, c(0,1),tol=0.0001)$minimum
if ((xx2 - X[2*i-2]) > delta1)
xx2<-X[2*i-2]+delta1

yy1 = optimize(f=function(y){abs(pdlt(medrh00,medrh01,medrh10,medeta,X[2*i-3],y) - theta)}, c(0,1),tol=0.0001)$minimum
if ((yy1 - Y[2*i-3]) > delta1)
yy1<-Y[2*i-3]+delta1

yy2 = optimize(f=function(y){abs(pdlt(medrh00,medrh01,medrh10,medeta,X[2*i-2],y) - theta)}, c(0,1),tol=0.0001)$minimum
if ((yy2 - Y[2*i-2]) > delta1)
yy2<-Y[2*i-2]+delta1


if (X[2*i-3] == X[2*i-5]) {

X<-c(X,xx1,X[2*i-2])
Y<-c(Y,Y[2*i-3],yy2)} else {
X<-c(X,X[2*i-3],xx2)
Y<-c(Y,yy1,Y[2*i-2])}


n<-n+2
zz1<-resp_loglog(trho00,trho01,trho10,teta,theta,X[2*i-1],Y[2*i-1])
zz2<-resp_loglog(trho00,trho01,trho10,teta,theta,X[2*i],Y[2*i])
Z<-c(Z,zz1,zz2)}

dosex[kk,]<-X
dosey[kk,]<-Y
dlt[kk,]<-Z


vrho00[kk]<-median(ss$rho00)
vrho01[kk]<-median(ss$rho01)
vrho10[kk]<-median(ss$rho10)
veta[kk]<-median(ss$eta)

}

tempost<-numeric()

for (i in 1 :M) {

tempost[i]<-max(postlow[i,]) }

pstop<-length(tempost[tempost > delta]) / M

# output
write.table(vrho00,"rho00.txt")
write.table(vrho01,"rho01.txt")
write.table(vrho10,"rho10.txt")
write.table(veta,"eta.txt")
write.table(dosex,"dosex.txt")
write.table(dosey,"dosey.txt")
write.table(dlt,"dlt.txt")
write.table(postlow,"postlow.txt")
