
# Define working directory
setwd("L:/Shared/SH_LiQ/JSM_2016/ThreePatients/continuous/probit")

# Clean the workspace and invoke useful libraries
rm(list=ls())
library(rjags)
library(plotrix)

###################################################################################################################
# This program simulates M trials for dose finding of combination of two agents. It uses the logistic model
# logit(p)=alpha+beta*x+gamma*y + eta*x*y.
# Dose levels x and y are standardized to be between 0 and 1.
# The model is reparameterized in terms of rho00 = probability of DLT when x=y=0,
# rho01 = probability of DLT when when x=0 y=1, rho10 = probability of DLT when x = 1 y = 0, and eta = interaction term between # drugs.
# Feasibility bound alpha is variable
# MTD is truncated to interval (-Xmin / (Xmax - Xmin), infinity) where Xmin and Xmax are actual dose levels of both agent A and # agent B.
# A dose escalation jump is restricted to be no more than delta1 = 20% or 25% of dose range.
###################################################################################################################
# This function simulates a binary DLT response.
resplogistcomb<-function(rho00,rho01,rho10,eta,theta,x,y){
 alpha<-log(rho00/(1-rho00))
 beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
 gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
 p<-1/(1+exp(-alpha-beta*x-gamma*y-eta*x*y))
 u<-runif(1)
 if (u < p)
 z<-1
 else
 z<-0
 z}

resp_normal<-function(rho00,rho01,rho10,eta,sigma,theta,x,y){
  alpha<-log(rho00/(1-rho00))
  beta <-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
  gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
  alpha1<-alpha-log(theta/(1-theta))+sigma*qnorm(theta,0,1)
  aa<-(alpha1+beta*x+gamma*y+eta*x*y)/sigma
  p<-pnorm(aa,0,1)
  u<-runif(1)
  if (u < p)
    z<-1
  else
    z<-0
  z}

# This function computes the probability of DLT at dose combination (x,y)

pdlt<-function(rho00,rho01,rho10,eta,x,y){
alpha<-log(rho00/(1-rho00))
 beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
 gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
p<-1/(1+exp(-alpha-beta*x-gamma*y-eta*x*y))
p}



# This function finds the set of MTD
twodimmtd2<-function(rho00,rho01,rho10,eta,theta,x){
alpha<-log(rho00/(1-rho00))
beta<-log(rho10/(1-rho10)) - log(rho00/(1-rho00))
gamma<-log(rho01/(1-rho01)) - log(rho00/(1-rho00))
a<-(log(theta/(1-theta))-alpha-beta*x)/(gamma+eta*x)
a}

delta1<-0.2
Xlow<-0
Ylow<-0

Xmin<-100
Xmax<-500
lb<--Xmin/(Xmax - Xmin)

# Threshold for posterior probability of toxicity at minimum dose for stopping trial
delta<-0.8

# Number of Trials
M<-1000
# True parameters
source("./simu_b/trueparameters.r")


# Target Probability of DLT 
theta<-0.33333
sigma=2

# Number of patients in a trial
NN<-42
# Feasibility bound
alpha<-numeric()
alphaa<-seq(0.25,0.5,by=0.05)
for (ind in 1:5){
alpha[ind]<-alphaa[ind]
}
for (ind in 6:(NN-1)){
alpha[ind]<-alphaa[6]
}
# Prior parameters for rho00, rho01, rho10, eta
a00<-1
b00<-1
a01<-1
b01<-1
a10<-1
b10<-1
# Prior parameters for interaction term eta
a<-0.8
b<-0.0384

vrho00<-numeric()
veta<-numeric()
vrho01<-numeric()
vrho10<-numeric()

# mcmc parameters

chains<-1
burn<-10000
mm<-5000

# Declaration of intermediate statistics
trcmtdx1<-numeric()
trcmtdx2<-numeric()
trcmtdy1<-numeric()
trcmtdy1<-numeric()

# Declaration of the output statistics
dosex<-mat.or.vec(M,NN)
dosey<-mat.or.vec(M,NN)
dlt<-mat.or.vec(M,NN)
postlow<-mat.or.vec(M, NN/3)
temp1<-numeric()
temp2<-numeric()

X<-matrix(NA, NN/3, 3)
Y<-matrix(NA, NN/3, 3)
Z<-matrix(NA, NN/3, 3)

# Start of main loop for the trials

for (kk in 1:M) {

# data for first cohort of two patients- Z is DLT status


X[1, ] <- c(0,0,0)
Y[1, ] <- c(0,0,0)

# Simulate responses for patients 1 and 2 from logistic model

zz1<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[1, 1],Y[1, 1])
zz2<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[1, 2],Y[1, 2])
zz3<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[1, 3],Y[1, 3])
Z[1, ] <- c(zz1,zz2,zz3)
n <-3

# Get posterior median of model parameters

j=jags.model('combination_cohort_2.bug.txt',
             data=list('Z'=as.numeric(t(Z)),'X'=c(t(X)),'Y'=c(t(Y)),
                       'a00'=a00,'b00'=b00,'a01'=a01,'b01'=b01,'a10'=a10,'b10'=b10,'a'=a,'b'=b,'N'=n),
             n.chains=chains,n.adapt=burn)

s=coda.samples(j,c('rho00','rho01','rho10','eta'),mm)
ss=as.data.frame(s[[1]])

medrh00<-median(ss$rho00)
medrh01<-median(ss$rho01)
medrh10<-median(ss$rho10)
medeta<-median(ss$eta)

####

##
x.index <- 1
y.index <- 2

xx = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,Y[1, x.index]) - theta)}, c(0,1),tol=0.0001)$minimum
if ((xx - X[1, x.index ]) > delta1)
  xx<-X[1, x.index]+delta1
X[2, x.index] <- xx
X[2, y.index] <- X[1, y.index]

yy = optimize(f=function(y){abs(pdlt(medrh00,medrh01,medrh10,medeta,X[1, y.index],y) - theta)}, c(0,1),tol=0.0001)$minimum
if ((yy - Y[1, y.index ]) > delta1)
  yy<-Y[1, y.index]+delta1
Y[2, y.index] <- yy
Y[2, x.index] <- Y[1, x.index]

#Find dose xx1 in interval [0,1] so that |pdlt(medrho00,medrho01,medrho10,medeta,xx1,Y[1]) - theta| is smallest
xx = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,x) - theta)}, c(0,1),tol=0.0001)$minimum
yy <- xx

if ((xx - X[1,3]) > delta1)
  xx <- X[1, 3]+delta1
if ((yy - Y[1,3]) > delta1)
  yy <- Y[1, 3]+delta1

# Update data

X[2, 3] <- xx
Y[2, 3] <- yy

# Simulate responses for patients 3 and 4 from logistic model

zz1<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[2, 1],Y[2, 1])
zz2<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[2, 2],Y[2, 2])
zz3<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[2, 3],Y[2, 3])
Z[2, ]<-c(zz1,zz2,zz3)
n<-6

for (i in 3:(NN/3)) {
j=jags.model('combination_cohort_2.bug.txt',data=list('Z'=as.numeric(t(Z)),'X'=as.numeric(t(X)),'Y'=as.numeric(t(Y)),
                                                      'a00'=a00,'b00'=b00,'a01'=a01,'b01'=b01,'a10'=a10,'b10'=b10,'a'=a,'b'=b,'N'=n),n.chains=chains,n.adapt=burn)

s=coda.samples(j,c('rho00','rho01','rho10','eta'),mm)
ss=as.data.frame(s[[1]])

# Calculating posterior probability of toxicity at initial dose
for (ll in 1:mm) {

temp1[ll]<-pdlt(ss$rho00[ll],ss$rho01[ll],ss$rho10[ll],ss$eta[ll],Xlow, Ylow)

if (temp1[ll] > theta+0.05)
(temp2[ll]<-1) else
(temp2[ll]<-0)
}
postlow[kk,i]<-sum(temp2)/mm


medrh00<-median(ss$rho00)
medrh01<-median(ss$rho01)
medrh10<-median(ss$rho10)
medeta<-median(ss$eta)

##
x.index <- setdiff(1:2, x.index)
y.index <- setdiff(1:2, x.index)

xx = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,Y[(i-1), x.index]) - theta)}, c(0,1),tol=0.0001)$minimum
if ((xx - X[(i-1), x.index ]) > delta1)
  xx<-X[(i-1), x.index]+delta1
X[i, x.index] <- xx
X[i, y.index] <- X[(i-1), y.index]

yy = optimize(f=function(y){abs(pdlt(medrh00,medrh01,medrh10,medeta,X[(i-1), y.index],y) - theta)}, c(0,1),tol=0.0001)$minimum
if ((yy - Y[(i-1), y.index ]) > delta1)
  yy<-Y[(i-1), y.index]+delta1
Y[i, y.index] <- yy
Y[i, x.index] <- Y[(i-1), x.index]

#Find dose xx1 in interval [0,1] so that |pdlt(medrho00,medrho01,medrho10,medeta,xx1,Y[1]) - theta| is smallest
xx = optimize(f=function(x){abs(pdlt(medrh00,medrh01,medrh10,medeta,x,x) - theta)}, c(0,1),tol=0.0001)$minimum
yy <- xx

if ((xx - X[(i-1), 3]) > delta1)
  xx <- X[(i-1), 3]+delta1
if ((yy - Y[(i-1), 3]) > delta1)
  yy <- Y[(i-1), 3]+delta1

X[i, 3] <- xx
Y[i, 3] <- yy

n<-n+3
zz1<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[i, 1],Y[i, 1])
zz2<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[i, 2],Y[i, 2])
zz3<-resp_normal(trho00,trho01,trho10,teta,sigma,theta,X[i, 3],Y[i, 3])
Z[i, ]<-c(zz1,zz2,zz3)}

dosex[kk,]<- c(t(X))
dosey[kk,]<-c(t(Y))
dlt[kk,]<-c(t(Z))


vrho00[kk]<-median(ss$rho00)
vrho01[kk]<-median(ss$rho01)
vrho10[kk]<-median(ss$rho10)
veta[kk]<-median(ss$eta)

}

tempost<-numeric()

for (i in 1 :M) {

tempost[i]<-max(postlow[i,]) }

pstop<-length(tempost[tempost > delta]) / M

# output
write.table(vrho00,"rho00.txt")
write.table(vrho01,"rho01.txt")
write.table(vrho10,"rho10.txt")
write.table(veta,"eta.txt")
write.table(dosex,"dosex.txt")
write.table(dosey,"dosey.txt")
write.table(dlt,"dlt.txt")
write.table(postlow,"postlow.txt")

