library(MASS)
library(reshape2)

### Auxiliar functions

mtdy_logistic <-function(rho00, rho01, rho10, eta, theta, x){
  alpha <- qlogis(rho00)
  beta <- qlogis(rho10) - qlogis(rho00)
  gamma <- qlogis(rho01) - qlogis(rho00)
  a <- (qlogis(theta) - alpha - beta*x)/(gamma + eta*x)
  a
}

mtdx_logistic <-function(rho00, rho01, rho10, eta, theta, y){
  alpha <- qlogis(rho00)
  beta <- qlogis(rho10) - qlogis(rho00)
  gamma <- qlogis(rho01) - qlogis(rho00)
  a <- (qlogis(theta) - alpha - gamma*y)/(beta + eta*y)
  a
}

getLevel <- function(x,y,prob=0.95) {
  kk <- MASS::kde2d(x,y)
  dx <- diff(kk$x[1:2])
  dy <- diff(kk$y[1:2])
  sz <- sort(kk$z)
  c1 <- cumsum(sz) * dx * dy
  approx(c1, sz, xout = 1 - prob)$y
}

### Simulation setup

nd <- 3
ns <- 4
nm <- 4

design <- rep(c("crm2p", "crm3p", "ewoc2p"), each = ns*nm)
scenario <- rep(c("a", "b", "c", "d"), times = nd*nm)
model <- rep(rep(c("logit", "loglog", "probit", "normal"), each = ns), 
             times = nd)

### Summary

file <- paste0(design, "_nip_", model, "_s", scenario, ".Rdata")

for (f in 1:length(file)) {
  
  load(paste0("M:/DinizMA/Research/Drug Combination/CRM/results/continuous/",
              file[f]))  
  
  lower_x <- mtdx_logistic(trho00, trho01, trho10, teta, theta, 1)
  upper_x <- mtdx_logistic(trho00, trho01, trho10, teta, theta, 0)
  
  xgrid <- seq(max(0, lower_x), min(upper_x, 1), length = 100)
  ymtd <- mtdy_logistic(trho00, trho01, trho10, teta, theta, xgrid)
  
  fdist <- function(x,pt,rho00,rho01,rho10,eta,theta){
    
    alpha <- log(rho00/(1-rho00))
    beta <- log(rho10/(1-rho10)) - log(rho00/(1-rho00))
    gamma <- log(rho01/(1-rho01)) - log(rho00/(1-rho00))
    
    d <- (pt[1] - x)^2 + 
      (pt[2] - (qlogis(theta) - alpha - beta*x)/(gamma + eta*x))^2
    out <- sqrt(d)
    return(out)
  }
  
  pointwise <- matrix(NA, ncol = 100, nrow = M)
  estimate <- matrix(NA, ncol = 100, nrow = M)
  
  for (i in 1:M){
    for (j in 1:100){
      
      opt <- optimize(f = fdist,c(0,1), tol=0.0001, c(xgrid[j], ymtd[j]), 
                      vrho00[i], vrho01[i], vrho10[i], veta[i], 
                      theta)
      
      pointwise[i, j] <- opt$objective
      
      if (ymtd[j] > twodimmtd2(vrho00[i],vrho01[i],vrho10[i], veta[i],
                               theta,  opt$minimum)) 
        pointwise[i, j] <- -pointwise[i, j]
    }
    
    estimate[i, ] <- twodimmtd2(vrho00[i],vrho01[i],vrho10[i], veta[i], 
                                theta, xgrid)
  }
  
  percent10 <- rep(NA, 100)
  percent20 <- rep(NA, 100)
  
  for (j in 1:100){
    cond10 <- pointwise[, j] <= 0.1*sqrt(xgrid[j]^2 + ymtd[j]^2)
    cond20 <- pointwise[, j] <= 0.2*sqrt(xgrid[j]^2 + ymtd[j]^2)
    percent10[j] <- (length(pointwise[, j][cond10])/M)*100
    percent20[j] <- (length(pointwise[, j][cond20])/M)*100
  }
  
  temp01 <- data.frame(design = design[f], model = model, 
                       scenario = scenario[f], bias = colMeans(pointwise),
                       percent10 = percent10, percent20 = percent20,
                       xgrid = xgrid, ymtd_true = ymtd, 
                       ymtd_estimate = colMeans(estimate))
  
  if (model == "logit" & design[f] == "crm2p"){
    L90 <- getLevel(vdosex[, 40], vdosey[, 40], prob = 0.9)
    kde <- kde2d(as.numeric(vdosex[, 40]), as.numeric(vdosey[, 40]))
    dimnames(kde$z) <- list(kde$x, kde$y)
    temp02 <- data.frame(design = design[f], model = model, 
                         scenario = scenario[f], melt(kde$z), L90 = L90)
    temp04 <- data.frame(design = design[f], model = model, 
                         scenario = scenario[f], x = vdosex[, 40], y = vdosey[, 40])
  }
  
  temp03 <-  data.frame(design = design[f], model = model, 
                        scenario = scenario[f], 
                        dlt = round(100*mean(rowMeans(vdlt)), 2),
                        dltg10 = round(100*mean(rowMeans(vdlt) > 0.43), 2))

  if (f > 1){
    temp01 <- rbind(data_efficacy, temp01)
    if (model == "logit" & design[f] == "crm2p"){
      temp02 <- rbind(data_contour, temp02)
      temp04 <- rbind(data_points, temp04)
    }
    temp03 <- rbind(data_safety, temp03)

  }
  
  data_efficacy <- temp01
  if (model == "logit" & design[f] == "crm2p"){
    data_contour <- temp02
    data_points <- temp04
  }
  data_safety <- temp03
}

save.image("M:/DinizMA/Research/Drug Combination/CRM/opc_continuous_output.Rdata")